<?php

if(!defined('_PS_VERSION_')) exit;

class Ratepanel extends Module
{
    const CONFIG_NAME_API_KEY = 'RATEPANEL_API_KEY';
    const CONFIG_NAME_TRIGGER_STATUS_ID = 'RATEPANEL_TRIGGER_STATUS_ID';
    const CONNECTION_STATUS_CANT_CONNECT = 'cant_connect';
    const CONNECTION_STATUS_INVALID_KEY = 'invalid_key';
    const CONNECTION_STATUS_SUCCESS = 'success';
    private $langId = null;

    public function __construct()
    {
        // Get default language
        $this->langId = (int)Configuration::get('PS_LANG_DEFAULT');

        $this->name = 'ratepanel';
        $this->version = '1.0.4';
        $this->author = 'Ratepanel.com';
        $this->tab = 'other';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = array('min' => '1.5', 'max' => _PS_VERSION_);
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Ratepanel API');
        $this->description = $this->l('Module to connect your PrestaShop to your Ratepanel business account.');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
    }

    public function install()  {
        Configuration::updateValue(Ratepanel::CONFIG_NAME_TRIGGER_STATUS_ID, 4);

        if (parent::install() == false OR !$this->registerHook('postUpdateOrderStatus'))
            return false;
        return true;
    }

    public function uninstall()  {
        if (parent::uninstall() == false OR !$this->unregisterHook('postUpdateOrderStatus'))
            return false;
        return true;
    }

    public function getContent()
    {
        $output = null;

        if (Tools::isSubmit('submit'.$this->name))
        {
            // Update settings
            $valKey = strval(Tools::getValue(Ratepanel::CONFIG_NAME_API_KEY));
            $valStatus = strval(Tools::getValue(Ratepanel::CONFIG_NAME_TRIGGER_STATUS_ID));

            if (!$valKey || !$valStatus)
                $output .= $this->displayError($this->l('Invalid Configuration value'));
            else
            {
                Configuration::updateValue(Ratepanel::CONFIG_NAME_API_KEY, $valKey);
                Configuration::updateValue(Ratepanel::CONFIG_NAME_TRIGGER_STATUS_ID, $valStatus);
                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }
        return $output.$this->displayForm();
    }

    public function displayForm()
    {
        //Get status
        $status = $this->getConnectionStatus();
        switch($status)
        {
            case Ratepanel::CONNECTION_STATUS_CANT_CONNECT:
                $statusMsg = 'Can not connect to Ratepanel.com';
                break;

            case Ratepanel::CONNECTION_STATUS_INVALID_KEY:
                $statusMsg = 'The API key is invalid';
                break;

            case Ratepanel::CONNECTION_STATUS_SUCCESS:
                $statusMsg = 'Success. API key valid';
                break;
        }

        // Init Fields form array
        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Settings'),
            ),
            'input' => array(
                array(
                    'type'      => 'text',
                    'label'     => $this->l('API key'),
                    'name'      => Ratepanel::CONFIG_NAME_API_KEY,
                    'required'  => true,
                    'desc'      => $this->l('Can be found on the settings page in the Ratepanel admin.')
                ),
                array(
                    'type'      => 'select',
                    'label'     => $this->l('Invite on order status'),
                    'desc'      => $this->l('This determine what the order status should be to send a Ratepanel invitation to the customer'),
                    'name'      => Ratepanel::CONFIG_NAME_TRIGGER_STATUS_ID,
                    'required'  => true,
                    'options'   => array(
                        'query' => $this->getStatuses(),
                        'id'    => 'id',
                        'name'  => 'name'
                    )
                ),
                array(
                    'type'      => 'free',
                    'label'     => 'Connection',
                    'desc'      => '<span class="text-' . ($status === Ratepanel::CONNECTION_STATUS_SUCCESS ? 'success' : 'danger') . '" style="font-style: normal; font-weight: bold;">' . $statusMsg . '</span>'
                )
            ),
            'submit' => array(
                'title' => $this->l('Save')
            )
        );

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        // Language
        $helper->default_form_language = $this->langId;
        $helper->allow_employee_form_lang = $this->langId;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = array(
            'save' =>
                array(
                    'desc' => $this->l('Save'),
                    'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                        '&token='.Tools::getAdminTokenLite('AdminModules'),
                ),
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        // Load current value
        $helper->fields_value[Ratepanel::CONFIG_NAME_API_KEY] = $this->getKey();
        $helper->fields_value[Ratepanel::CONFIG_NAME_TRIGGER_STATUS_ID] = $this->getTriggerStatusId();

        return $helper->generateForm($fields_form);
    }

    private function getKey()
    {
        return Configuration::get(Ratepanel::CONFIG_NAME_API_KEY);
    }

    private function getTriggerStatusId()
    {
        return Configuration::get(Ratepanel::CONFIG_NAME_TRIGGER_STATUS_ID);
    }

    private function getStatuses()
    {
        $sql = 'SELECT os.id_order_state AS id, osl.name AS name FROM '._DB_PREFIX_.'order_state os
                INNER JOIN '._DB_PREFIX_.'order_state_lang osl ON osl.id_order_state=os.id_order_state AND osl.id_lang=1';

        return DB::getInstance()->executeS($sql, true, 0);
    }

    private function getConnectionStatus()
    {
        $key = $this->getKey();

        if(!preg_match('/^key_[0-9a-z]{32}$/i', $key))
            return Ratepanel::CONNECTION_STATUS_INVALID_KEY;

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_POST, 0);
        curl_setopt($curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($curl, CURLOPT_USERPWD, 'api:' . $this->getKey());
        curl_setopt($curl, CURLOPT_URL, 'https://api.ratepanel.com');
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);

        $result = curl_exec($curl);
        curl_close($curl);

        if($result)
        {
            //Decode JSON
            $result = json_decode($result);

            if($result->error)
                return Ratepanel::CONNECTION_STATUS_INVALID_KEY;
            else
                return Ratepanel::CONNECTION_STATUS_SUCCESS;

        }else
            return Ratepanel::CONNECTION_STATUS_CANT_CONNECT;

    }

    public function hookPostUpdateOrderStatus($params)
    {
        $updatedStatus = $params['newOrderStatus']->id;
        if($updatedStatus === (int) $this->getTriggerStatusId())
        {
            $order = new Order((int) $params['id_order']);
            $customer = new Customer((int) $order->id_customer);

            $curl = curl_init();
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
            curl_setopt($curl, CURLOPT_USERPWD, 'api:' . $this->getKey());
            curl_setopt($curl, CURLOPT_URL, 'https://api.ratepanel.com/customer');
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, array(
                'name'          => $customer->firstname . ' ' . $customer->lastname,
                'email'         => $customer->email,
                'order_number'  => $order->id_cart,
                'invite'        => 1
            ));

            $result = curl_exec($curl);
            curl_close($curl);
        }
    }
}